@@ -1,22 +1,36 @@
+# -*- coding=utf -*-
+"""SQL Denormalized browser
+
+Note: This browser will become obsolete once the star browser is implemented.
+For more information about the upcoming browser see:
+
+* star_browser.py
+* in mapper.py see AttributeMapper and JoinFinder
+"""
+
+# FIXME: rename this to denormalized so backend name is "sql.denormalized"
+
 import cubes.browser
-import base
+import mapper
 import logging
 import cubes.model
 import collections
+from StringIO import StringIO
+
 try:
     from collections import OrderedDict
 except ImportError:
     from ordereddict import OrderedDict
     
-from cubes.common import logger_name
+from cubes.common import logger_name, get_logger
 from collections import deque
 
 try:
     import sqlalchemy
     import sqlalchemy.sql.expression as expression
     import sqlalchemy.sql.functions as functions
-except:
-    from cubes.util import MissingPackage
+except ImportError:
+    from cubes.common import MissingPackage
     _missing = MissingPackage("sqlalchemy", "Built-in SQL aggregation browser")
     sqlalchemy = expression = functions = _missing
 
@@ -98,7 +112,7 @@ class SQLBrowser(cubes.browser.AggregationBrowser):
         
         self.fact_key = cube.key
         if not self.fact_key:
-            self.fact_key = base.DEFAULT_KEY_FIELD
+            self.fact_key = mapper.DEFAULT_KEY_FIELD
 
         if connection is not None:
             # FIXME: This reflection is somehow slow (is there another way how to do it?)
@@ -282,7 +296,7 @@ class CubeQuery(object):
         self.cube = cell.cube
         self.cube_key = self.cube.key
         if not self.cube_key:
-            self.cube_key = base.DEFAULT_KEY_FIELD
+            self.cube_key = mapper.DEFAULT_KEY_FIELD
 
         self.key_column = self.view.c[self.cube_key]
 
@@ -705,62 +719,153 @@ class CubeQuery(object):
 # Slicer server - backend handling
 #
 
-# Backward compatibility - use [db] section in slicer configuration
-config_section = "db"
+def ddl_for_model(url, model, fact_prefix=None, dimension_prefix=None, schema_type=None):
+    """Create a star schema DDL for a model.
+    
+    Parameters:
+    
+    * `url` - database url – no connection will be created, just used by 
+       SQLAlchemy to determine appropriate engine backend
+    * `cube` - cube to be described
+    * `dimension_prefix` - prefix used for dimension tables
+    * `schema_type` - ignored in this backend
+    
+    As model has no data storage type information, following simple rule is
+    used:
+    
+    * fact ID is an integer
+    * all keys are strings
+    * all attributes are strings
+    * all measures are floats
+    
+    .. warning::
+    
+        Does not respect localized models yet.
+    
+    """
+    dim_tables = {}
+    dim_keys = {}
+    
+    out = StringIO()
 
-def create_workspace(model, config):
+    dimension_prefix = dimension_prefix or ""
+    fact_prefix = fact_prefix or ""
+    def dump(sql, *multiparams, **params):
+        out.write(('%s' %
+            sql.compile(dialect=engine.dialect)).strip()+';\n/\n')
+
+    engine = sqlalchemy.create_engine(url, strategy='mock', executor=dump)
+
+    metadata = sqlalchemy.MetaData(engine)
+
+    # Create dimension tables
+    
+    for dim in model.dimensions:
+        # If the dimension is represented by one field only, then there is
+        # no need to create a separate table.
+        if dim.is_flat and not dim.has_details:
+            continue
+        
+        # Create and store constructed table name and key identifier. They
+        # will be used in fact table creation
+        
+        name = dimension_prefix+dim.name
+        dim_tables[dim.name] = name
+        dim_key = dim.name + "_key"
+        dim_keys[dim.name] = dim_key
+
+        table = sqlalchemy.Table(name, metadata)
+        table.append_column(sqlalchemy.Column(dim_key, sqlalchemy.Integer, primary_key=True))
+        for attr in dim.all_attributes():
+            table.append_column(sqlalchemy.Column(attr.name, sqlalchemy.String))
+        
+    # Create fact tables
+
+    for cube in model.cubes.values():
+
+        table = sqlalchemy.Table(fact_prefix+cube.name, metadata)
+        table.append_column(sqlalchemy.Column(attr.name, sqlalchemy.Integer))
+
+        for dim in cube.dimensions:
+            if dim.is_flat and not dim.has_details:
+                col = sqlalchemy.Column(dim.name, sqlalchemy.String)
+            else:
+                fkey = "%s.%s" % (dim_tables[dim.name], dim_keys[dim.name])
+                col = sqlalchemy.Column(dim_keys[dim.name], sqlalchemy.Integer, sqlalchemy.ForeignKey(fkey), nullable=False)
+            table.append_column(col)
+    
+        for attr in cube.details:
+            table.append_column(sqlalchemy.Column(attr.name, sqlalchemy.String))
+            
+        for attr in cube.measures:
+            table.append_column(sqlalchemy.Column(attr.name, sqlalchemy.Float))
+    
+    metadata.create_all()
+    
+    return out.getvalue()
+
+
+def create_workspace(model, **options):
     """Create workspace for `model` with configuration in dictionary `config`. 
     This method is used by the slicer server."""
 
     try:
-        dburl = config["url"]
+        dburl = options["url"]
     except KeyError:
         raise Exception("No URL specified in configuration")
 
-    schema = config.get("schema")
-    view_prefix = config.get("view_prefix")
-    view_suffix = config.get("view_suffix")
+    schema = options.get("schema")
+    view_prefix = options.get("view_prefix")
+    view_suffix = options.get("view_suffix")
 
     engine = sqlalchemy.create_engine(dburl)
 
     workspace = SQLWorkspace(model, engine, schema, 
-                                    name_prefix = view_prefix,
-                                    name_suffix = view_suffix)
+                                    view_prefix = view_prefix,
+                                    view_suffix = view_suffix)
 
     return workspace
 
 class SQLWorkspace(object):
     """Factory for browsers"""
-    def __init__(self, model, engine, schema = None, name_prefix = None, name_suffix = None):
+    def __init__(self, model, engine, schema=None, view_prefix=None, 
+                 view_suffix=None):
         """Create a workspace"""
         super(SQLWorkspace, self).__init__()
         self.model = model
         self.engine = engine
-        self.metadata = sqlalchemy.MetaData(bind = self.engine)
-        self.name_prefix = name_prefix
-        self.name_suffix = name_suffix
+        self.metadata = sqlalchemy.MetaData(bind=self.engine)
+        self.view_prefix = view_prefix
+        self.view_suffix = view_suffix
         self.views = {}
         self.schema = schema
+
+        self.logger = cubes.get_logger()
         
-    def browser_for_cube(self, cube, locale = None):
+    def browser_for_cube(self, cube, locale=None):
         """Creates, configures and returns a browser for a cube"""
+        self.logger.info("workspace.create_browser() is depreciated, use "
+                         ".browser() instead")
+
+        return self.browser(cube, locale)
+
+    def browser(self, cube, locale=None):
         cube = self.model.cube(cube)
         view = self._view_for_cube(cube)
-        browser = SQLBrowser(cube, view = view, locale = locale)
+        browser = SQLBrowser(cube, view=view, locale=locale)
         return browser
         
-    def _view_for_cube(self, cube, view_name = None):
+    def _view_for_cube(self, cube, view_name=None):
         if cube.name in self.views:
             view = self.views[cube.name]
         else:
+            if cube.options:
+                view_name = cube.options.get("denormalized_view") or view_name
+                
             if not view_name:
-                if self.name_prefix:
-                    view_name = self.name_prefix
-                else:
-                    view_name = ""
-                view_name += cube.name
-                if self.name_suffix:
-                    view_name += self.name_suffix
+                prefix = self.view_prefix or ""
+                suffix = self.view_suffix or ""
+                view_name = "%s%s%s" % (prefix, cube.name, suffix)
 
             view = sqlalchemy.Table(view_name, self.metadata, autoload = True, schema = self.schema)
             self.views[cube.name] = view            
